@@ -72,7 +72,7 @@ class StreamHub(BaseHub, ABC):
     #
 
     # noinspection PyMethodMayBeStatic
-    def _create_channel(self, sock: socket.socket, remote: Optional[Address], local: Optional[Address]) -> Channel:
+    def _create_channel(self, remote: Optional[Address], local: Optional[Address], sock: socket.socket) -> Channel:
         """ create channel with socket & addresses """
         return StreamChannel(remote=remote, local=local, sock=sock)
 
@@ -82,22 +82,22 @@ class StreamHub(BaseHub, ABC):
         return self.__channel_pool.items
 
     # Override
-    def _remove_channel(self, channel: Channel):
+    def _remove_channel(self, remote: Optional[Address], local: Optional[Address], channel: Optional[Channel]):
         """ remove cached channel """
-        self.__channel_pool.remove(remote=channel.remote_address, local=channel.local_address, item=channel)
+        self.__channel_pool.remove(remote=remote, local=local, item=channel)
 
     def _get_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
         """ get cached channel """
         return self.__channel_pool.get(remote=remote, local=local)
 
-    def _set_channel(self, channel: Channel):
+    def _set_channel(self, remote: Optional[Address], local: Optional[Address], channel: Channel):
         """ cache channel """
-        self.__channel_pool.set(remote=channel.remote_address, local=channel.local_address, item=channel)
+        self.__channel_pool.set(remote=remote, local=local, item=channel)
 
     # Override
     def open(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
         assert remote is not None, 'remote address empty: %s, %s' % (remote, local)
-        return self.__channel_pool.get(remote=remote, local=None)
+        return self._get_channel(remote=remote, local=local)
 
 
 class ServerHub(StreamHub, Runnable):
@@ -115,13 +115,13 @@ class ServerHub(StreamHub, Runnable):
         return self.__master
 
     def _set_master(self, master: socket.socket):
-        # 1. check old socket
-        old = self._get_master()
+        # 1. replace with new socket
+        old = self.__master
+        self.__master = master
+        # 2. close old socket
         if old is not None and old is not master:
             if is_opened(sock=old):
                 old.close()
-        # 2. set new socket
-        self.__master = master
 
     @property
     def local_address(self) -> Optional[Address]:
@@ -136,9 +136,9 @@ class ServerHub(StreamHub, Runnable):
         return self.__running
 
     # Override
-    def _create_connection(self, channel: Channel, remote: Address, local: Optional[Address]) -> Optional[Connection]:
+    def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
         gate = self.delegate
-        conn = BaseConnection(remote=remote, local=None, channel=channel, delegate=gate)
+        conn = BaseConnection(remote=remote, local=local, channel=channel, delegate=gate)
         conn.start()  # start FSM
         return conn
 
@@ -174,27 +174,27 @@ class ServerHub(StreamHub, Runnable):
             try:
                 master = self._get_master()
                 sock, address = master.accept()
-                self._accept(sock=sock, remote=address, local=self.local_address)
+                self._accept(remote=address, local=self.local_address, sock=sock)
             except socket.error as error:
                 print('[TCP] socket error: %s' % error)
             except Exception as error:
                 print('[TCP] accept error: %s' % error)
 
-    def _accept(self, sock: socket.socket, remote: Address, local: Address):
+    def _accept(self, remote: Address, local: Address, sock: socket.socket):
         # override for user-customized channel
         assert sock is not None, 'socket error: %s, %s' % (remote, local)
-        channel = self._create_channel(sock=sock, remote=remote, local=local)
+        channel = self._create_channel(remote=remote, local=local, sock=sock)
         assert channel is not None, 'failed to create socket channel: %s, remote=%s, local=%s' % (sock, remote, local)
-        self._set_channel(channel=channel)
+        self._set_channel(remote=channel.remote_address, local=channel.local_address, channel=channel)
 
 
 class ClientHub(StreamHub):
     """ Stream Client Hub """
 
     # Override
-    def _create_connection(self, channel: Channel, remote: Address, local: Optional[Address]) -> Optional[Connection]:
+    def _create_connection(self, remote: Address, local: Optional[Address], channel: Channel) -> Optional[Connection]:
         gate = self.delegate
-        conn = ActiveConnection(remote=remote, local=None, channel=channel, delegate=gate, hub=self)
+        conn = ActiveConnection(remote=remote, local=local, channel=channel, delegate=gate, hub=self)
         conn.start()  # start FSM
         return conn
 
@@ -204,7 +204,7 @@ class ClientHub(StreamHub):
         if channel is None:  # and remote is not None:
             channel = self.__create_socket_channel(remote=remote, local=local)
             if channel is not None:
-                self._set_channel(channel=channel)
+                self._set_channel(remote=channel.remote_address, local=channel.local_address, channel=channel)
         return channel
 
     def __create_socket_channel(self, remote: Optional[Address], local: Optional[Address]) -> Optional[Channel]:
@@ -213,7 +213,7 @@ class ClientHub(StreamHub):
             sock = create_socket(remote=remote, local=local)
             if local is None:
                 local = get_local_address(sock=sock)
-            return self._create_channel(sock=sock, remote=remote, local=local)
+            return self._create_channel(remote=remote, local=local, sock=sock)
         except socket.error as error:
             print('[TCP] failed to create channel %s -> %s: %s' % (local, remote, error))
 
